﻿--[[--
	alex@0
--]]--
----------------------------------------------------------------------------------------------------
local ADDON, ns = ...;
----------------------------------------------------------------------------------------------------
local math, table, string, pairs, type, select, tonumber, unpack = math, table, string, pairs, type, select, tonumber, unpack;
local min, max = min, max;
local tinsert, tremove = tinsert, tremove;
local _G = _G;
local GameTooltip = GameTooltip;
----------------------------------------------------------------------------------------------------
local GUID = UnitGUID('player');
local saved_sets = {  };
local L = setmetatable({  }, { __newindex = function(t, k, v) rawset(t, k, (v == true) and k or v); end});
local LOCALE = GetLocale();
if LOCALE == 'zhCN' or LOCALE == 'zhTW' then
	L["INVENTORY_IS_FULL"] = "背包已满！";
	L["IN_COMBAT"] = "战斗状态中无法换装！";
	L["Add a new outfit"] = "添加一个套装";
	L["Delete this outfit?"] = "删除此套装？";
	L["OK"] = "确定";
	L["Cancel"] = "取消";
	L["Save"] = "保存";
	L["Equip"] = "装备";
	L["Style"] = "风格";
	L["Style_TC"] = "文字+图标风格";
	L["Style_T"] = "图标风格";
	L["Style_C"] = "文字风格";
	L["Take_Off_All"] = "一键脱光";
	L["Take_Off_All_ButtonText"] = "脱";
	L["Take-off-all On Right"] = "一键脱光按钮在右边";
	L["Take-off-all On Left"] = "一键脱光按钮在左边";
	L["takeoffAll_include_finger_and_trinket"] = "一键脱光\124cffff0000包括\124r戒指饰品披风和耐久度为0的装备";
	L["takeoffAll_include_finger_and_trinket_false"] = "一键脱光\124cffff0000不包括\124r戒指饰品披风和耐久度为0的装备";
	L["CTRL-DRAG-TO-MOVE"] = "\124cffff40ff按住ctrl拖动来移动位置\124r";
	L["WAITING_FOR_REGEN_ENABLED"] = "战斗结束时更新";
elseif LOCALE == "ruRU" then
	L["INVENTORY_IS_FULL"] = "Инвентарь полон";
	L["IN_COMBAT"] = "Не может измениться в бою!";
	L["Add a new outfit"] = "Добавить костюм";
	L["Delete this outfit?"] = "Удалить этот костюм?";
	L["OK"] = "Хорошо";
	L["Cancel"] = "Отмена";
	L["Save"] = "Сохранить";
	L["Equip"] = "Экипировка";
	L["Style"] = "Стиль";
	L["Style_TC"] = "Стиль текста и значков";
	L["Style_T"] = "Стиль иконок";
	L["Style_C"] = "Стиль текста";
	L["Take_Off_All"] = "Снять все снаряжение";
	L["Take_Off_All_ButtonText"] = "С";
	L["Take-off-all On Right"] = "button of  \"Снять все снаряжение\" layed on the right";
	L["Take-off-all On Left"] = "button of  \"Снять все снаряжение\" layed on the left";
	L["takeoffAll_include_finger_and_trinket"] = "Снять все снаряжение включает в себя кольцо, акссесуар, плащ и снаряжение с продолжительностью 0%";
	L["takeoffAll_include_finger_and_trinket_false"] = "Снять все снаряжение \124cffff0000DONOT\124r включает в себя кольцо, акссесуар, плащ и снаряжение с продолжительностью 0%";
	L["CTRL-DRAG-TO-MOVE"] = "\124cffff40ffHolding ctrl and left-drag to move\124r";
	L["WAITING_FOR_REGEN_ENABLED"] = "Update after combat";
else
	L["INVENTORY_IS_FULL"] = "Inventory is full";
	L["IN_COMBAT"] = "In Combat";
	L["Add a new outfit"] = true;
	L["Delete this outfit?"] = true;
	L["OK"] = true;
	L["Cancel"] = true;
	L["Save"] = true;
	L["Equip"] = true;
	L["Style"] = true;
	L["Style_TC"] = "Text & Texture";
	L["Style_T"] = "Text Only";
	L["Style_C"] = "Texture Only";
	L["Take_Off_All"] = "Stripping";
	L["Take_Off_All_ButtonText"] = "S";
	L["Take-off-all On Right"] = "Stripping button On Right";
	L["Take-off-all On Left"] = "Stripping button On Left";
	L["takeoffAll_include_finger_and_trinket"] = "Stripping \124cffff0000DONOT INCLUDES\124r finger, trinket, cloak and duration 0% gear";
	L["takeoffAll_include_finger_and_trinket_false"] = "Stripping \124cffff0000DONOT INCLUDES\124r finger, trinket, cloak and duration 0% gear";
	L["CTRL-DRAG-TO-MOVE"] = "\124cffff40ffHolding ctrl and left-drag to move\124r";
	L["WAITING_FOR_REGEN_ENABLED"] = "Update after combat";
end
local _PaperDollItemSlotButton_OnEnter = PaperDollItemSlotButton_OnEnter;
local _PaperDollItemSlotButton_OnLeave = PaperDollItemSlotButton_OnLeave;
local SlotInfo = {
	[0] = { INVTYPE_AMMO = 1, },
	[1] = { INVTYPE_HEAD = 1, },
	[2] = { INVTYPE_NECK = 1, },
	[3] = { INVTYPE_SHOULDER = 1, },
	[4] = { INVTYPE_BODY = 1, },
	[5] = { INVTYPE_CHEST = 1, INVTYPE_ROBE = 1, },
	[6] = { INVTYPE_WAIST = 1, },
	[7] = { INVTYPE_LEGS = 1, },
	[8] = { INVTYPE_FEET = 1, },
	[9] = { INVTYPE_WRIST = 1, },
	[10] = { INVTYPE_HAND = 1, },
	[11] = { INVTYPE_FINGER = 1, },
	[12] = { INVTYPE_FINGER = 1, },
	[13] = { INVTYPE_TRINKET = 1, },
	[14] = { INVTYPE_TRINKET = 1, },
	[15] = { INVTYPE_CLOAK = 1, },
	[16] = { INVTYPE_WEAPONMAINHAND = 1, INVTYPE_2HWEAPON = 1, INVTYPE_WEAPON = 1, },
	[17] = { INVTYPE_WEAPON = 1, INVTYPE_WEAPONOFFHAND = 1, INVTYPE_SHIELD = 1, INVTYPE_HOLDABLE = 1, },
	[18] = { INVTYPE_RANGED = 1, INVTYPE_RANGEDRIGHT = 1, INVTYPE_THROWN = 1, INVTYPE_RELIC = 1, },
	[19] = { INVTYPE_TABARD = 1, },
}
local slot2Name = {
	[0] = "CharacterAmmoSlot",
	[1] = "CharacterHeadSlot",
	[2] = "CharacterNeckSlot",
	[3] = "CharacterShoulderSlot",
	[4] = "CharacterShirtSlot",
	[5] = "CharacterChestSlot",
	[6] = "CharacterWaistSlot",
	[7] = "CharacterLegsSlot",
	[8] = "CharacterFeetSlot",
	[9] = "CharacterWristSlot",
	[10] = "CharacterHandsSlot",
	[11] = "CharacterFinger0Slot",
	[12] = "CharacterFinger1Slot",
	[13] = "CharacterTrinket0Slot",
	[14] = "CharacterTrinket1Slot",
	[15] = "CharacterBackSlot",
	[16] = "CharacterMainHandSlot",
	[17] = "CharacterSecondaryHandSlot",
	[18] = "CharacterRangedSlot",
	[19] = "CharacterTabardSlot",
};
local name2Slot = {  };
for slot, name in pairs(slot2Name) do
	name2Slot[name] = slot;
end
local takeoffAll_order = {	-- by price
	16, 17, 18, 1, 5, 7, 3, 6, 8, 10, 9, 2, 11, 12, 13, 14, 15,--[[ 4,]]--[[ 19,]]
};
local NAME = "alaGearMan";
local ACTION_START = 144;

local function _error_(key, msg, ...)
	print("\124cffff0000" .. key .. "\124r", msg and "\124cffff0000" .. msg .. "\124r" or "", ...);
end

local COUNTING_TIME = 0.5;
local num_button_per_line = 6;
local buttonSize = 36;
local texture_unk = "Interface\\Icons\\inv_misc_questionmark";
local texture_takeoff = "Interface\\paperdollinfoframe\\ui-gearmanager-leaveitem-opaque";
local texture_ignore = "Interface\\paperdollinfoframe\\ui-gearmanager-undo";
local texture_add = "interface\\paperdollinfoframe\\character-plus";
local texture_glow = "Interface\\Buttons\\UI-ActionButton-Border";
local texture_glow_coord = { 0.25, 0.75, 0.25, 0.75, };
local texture_ignore_mask = "Interface\\paperdollinfoframe\\ui-gearmanager-leaveitem-transparent";
local texture_delete = "interface\\raidframe\\readycheck-notready";
local texture_modify = "interface\\scenarios\\scenarioicon-interact";
local texture_style_char = "interface\\buttons\\checkbuttonhilight";
----------------------------------------------------------------------------------------------------main
local _eventVehicle = CreateFrame("Frame");
_eventVehicle:RegisterEvent("ADDON_LOADED");
_eventVehicle:SetSize(4, 4);
_eventVehicle:EnableMouse(false);

--EquipItemByName(id/name/link, slot)
--RepairAllItems

local ui = { pdf_buttons = {  }, pdf_ignore_mask = {  },  };
local func = {  };
local var = {
	pdf_list = {  }, pdf_cur_slot = nil, pdf_cur_anchor = nil, pdf_ofs_ignore = nil, pdf_ofs_takeoff = nil,
	gm_cur_set = nil, 	--cur selected set / vary with delete action
	gm_prev_clicked_set = nil, gm_prev_clicked_time = -9999,
	gm_is_editing = nil, gm_editing_set = nil, gm_custom_icon_selected = nil, 
	gm_ignore = {  },
	cache = {  },
	bagItemCache = {  }, itemLoc = ItemLocation:CreateEmpty();
};
local global_var = {
	drop_table = {  },
};
local iconTable = {
	texture_unk, 
	136033, 132141, 132136, 132121, 132118, 134296, 136231, 132122, 132142, 132091, 132127, 132140, 132242, 132089, 
	132152, 132114, 132120, 132320, 132090, 132155, 132310, 132297, 132294, 132302, 132282, 132354, 132219, 132274, 
	136066, 132273, 132290, 136189, 132292, 132307, 132306, 132331, 136012, 132355, 134951, 132223, 132338, 136105, 
	--[[
		132363, 132938, 132350, 135358, 132340, 132353, 132316, 132366, 132357, 132337, 132333, 
		--druid
		16857, 6785, 6807, 99, 8998, 779, 5221, 1822, 9005, 22842, 22568, 1082, 5217, 5215, 1079, 5211, 1850,
		--hunter
		--mage
		--paladin
		--priest
		--rogue
		1784, 53, 1776, 6770, 703, 1966, 1943, 8676, 8647, 1766, 3420, 5763, 8681, 2835, 1752, 2098, 2983, 5171, 1856,
		--shaman
		--warlock
		--warrior 
		772, 78, 23881, 20252, 12294, 23922, 7384, 845, 6343, 7386, 6552, 694, 5308, 1464, 6572, 1715, 1160, 72, 100, 6673,
	--]]
};

function func.pdf_CreateButton(index)
	local button = CreateFrame("Button", nil, ui.pdf_menu);
	button:EnableMouse(true);
	button:SetSize(buttonSize, buttonSize);
	button:SetNormalTexture(texture_unk);
	button:SetPushedTexture(texture_unk);
	button:GetPushedTexture():SetVertexColor(0.5, 0.5, 0.5, 1.0);
	button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square");
	local glow = button:CreateTexture(nil, "OVERLAY");
	glow:SetAllPoints(true);
	glow:SetTexture(texture_glow);
	glow:SetBlendMode("ADD");
	glow:SetTexCoord(unpack(texture_glow_coord));
	-- glow:SetAlpha(0.75);
	glow:Show();
	button.glow = glow;
	button:SetScript("OnEnter", function(self, button)
		local id = self.id;
		GameTooltip:Hide();
		if id > #var.pdf_list then
		else
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
			GameTooltip:SetHyperlink(var.pdf_list[id]);
			GameTooltip:Show();
		end
		ui.pdf_menu.countingTime = nil;
	end);
	button:SetScript("OnLeave", function(self, button)
		if GameTooltip:IsOwned(self) then
			GameTooltip:Hide();
		end
		ui.pdf_menu.countingTime = COUNTING_TIME;
	end);
	button:SetScript("OnClick", function(self, button)
		local id = self.id;
		if pdf_ofs_ignore and id == #var.pdf_list + pdf_ofs_ignore then
			var.gm_ignore[var.pdf_cur_slot] = not var.gm_ignore[var.pdf_cur_slot];
			if var.gm_ignore[var.pdf_cur_slot] then
				ui.pdf_ignore_mask[var.pdf_cur_slot]:Show();
			else
				ui.pdf_ignore_mask[var.pdf_cur_slot]:Hide();
			end
		elseif pdf_ofs_takeoff and id == #var.pdf_list + pdf_ofs_takeoff then
			if not InCombatLockdown() or var.pdf_cur_slot == 16 or var.pdf_cur_slot == 17 or var.pdf_cur_slot == 18 then
				func.takeoff(var.pdf_cur_slot);
			end
		else
			if not InCombatLockdown() then
				func.equipItem(var.pdf_list[id], var.pdf_cur_slot);
				ui.pdf_menu.countingTime = - 1.0;
			end
		end
	end);
	ui.pdf_buttons[index] = button;
	button.id = index;
	return button;
end
function func.pdf_CreateMenu(direction)
	local hasInv = GetInventoryItemID('player', var.pdf_cur_slot) ~= nil;
	local nButtons = (hasInv and 1 or 0) + ((var.gm_is_editing or var.gm_cur_set) and 1 or 0) + #var.pdf_list
	if nButtons > #ui.pdf_buttons then
		for i = #ui.pdf_buttons + 1, nButtons do
			func.pdf_CreateButton(i);
		end
	elseif nButtons < #ui.pdf_buttons then
		for i = nButtons + 1, #ui.pdf_buttons do
			ui.pdf_buttons[i]:Hide();
		end
	end
	for i = 1, #var.pdf_list do
		local _, _, quality, _, _, _, _, _, _, t = GetItemInfo(var.pdf_list[i]);
		ui.pdf_buttons[i]:SetNormalTexture(t);
		ui.pdf_buttons[i]:SetPushedTexture(t);
		ui.pdf_buttons[i].glow:SetVertexColor(GetItemQualityColor(quality));
		ui.pdf_buttons[i].glow:Show();
		ui.pdf_buttons[i]:Show();
	end
	if hasInv then
		pdf_ofs_takeoff = 1;
		local i = #var.pdf_list + pdf_ofs_takeoff;
		ui.pdf_buttons[i]:SetNormalTexture(texture_takeoff);
		ui.pdf_buttons[i]:SetPushedTexture(texture_takeoff);
		-- ui.pdf_buttons[i].glow:SetVertexColor(1.0, 0.0, 0.0);
		ui.pdf_buttons[i].glow:Hide();
		ui.pdf_buttons[i]:Show();
	else
		pdf_ofs_takeoff = nil;
	end
	if var.gm_is_editing or var.gm_cur_set then
		pdf_ofs_ignore = (hasInv and 1 or 0) + 1;
		local i = #var.pdf_list + pdf_ofs_ignore;
		ui.pdf_buttons[i]:SetNormalTexture(texture_ignore);
		ui.pdf_buttons[i]:SetPushedTexture(texture_ignore);
		-- ui.pdf_buttons[i].glow:SetVertexColor(1.0, 0.0, 0.0);
		ui.pdf_buttons[i].glow:Hide();
		ui.pdf_buttons[i]:Show();
	else
		pdf_ofs_ignore = nil;
	end
	local pos = 2;
	local lines = 1;
	for i = 2, nButtons do
		if pos > num_button_per_line then
			ui.pdf_buttons[i]:SetPoint("TOP", ui.pdf_buttons[i - num_button_per_line], "BOTTOM", 0, -2);
			pos = 1;
			lines = lines + 1;
		else
			ui.pdf_buttons[i]:SetPoint("LEFT", ui.pdf_buttons[i - 1], "RIGHT", 2, 0);
			pos = pos + 1;
		end
	end
	if lines == 1 then
		ui.pdf_menu:SetSize(buttonSize * (pos - 1) + 2 * (pos - 2), buttonSize);
	else
		ui.pdf_menu:SetSize(buttonSize * num_button_per_line + 2 * (num_button_per_line - 2), buttonSize * lines + 2 * (lines - 1));
	end
	ui.pdf_menu:ClearAllPoints();
	if direction == 1 then	  --down
		if ui.pdf_menu:GetWidth() > var.pdf_cur_anchor:GetBottom() then
			ui.pdf_menu:SetPoint("BOTTOM", var.pdf_cur_anchor, "TOP", 0, 2);
		else
			ui.pdf_menu:SetPoint("TOP", var.pdf_cur_anchor, "BOTTOM", 0, -2);
		end
	elseif direction == 2 then  --slots in left
		if ui.pdf_menu:GetWidth() > var.pdf_cur_anchor:GetLeft() then
			ui.pdf_menu:SetPoint("LEFT", var.pdf_cur_anchor, "RIGHT", 2, 0);
		else
			ui.pdf_menu:SetPoint("RIGHT", var.pdf_cur_anchor, "LEFT", -2, 0);
		end
	elseif direction == 3 then  --slot in right
		if ui.pdf_menu:GetWidth() > GetScreenWidth() - var.pdf_cur_anchor:GetRight() then
			ui.pdf_menu:SetPoint("RIGHT", var.pdf_cur_anchor, "LEFT", -2, 0);
		else
			ui.pdf_menu:SetPoint("LEFT", var.pdf_cur_anchor, "RIGHT", 2, 0);
		end
	end
	ui.pdf_menu:Show();
	ui.pdf_menu.countingTime = nil;
end

function func.pdf_eventVehicle_OnUpdate(self, elasped)
	if var.pdf_cur_anchor and not var.pdf_cur_slot then
		if IsAltKeyDown() then
			local slot = name2Slot[var.pdf_cur_anchor:GetName()];
			if slot then
				--_PaperDollItemSlotButton_OnLeave(var.pdf_cur_anchor);
				wipe(var.pdf_list);
				for i = 0, 4 do
					for j = 1, GetContainerNumSlots(i) do
						local link = GetContainerItemLink(i, j);
						if link and IsEquippableItem(link) then
							local loc = select(9, GetItemInfo(link));
							if SlotInfo[slot][loc] then
								tinsert(var.pdf_list, link);
							end
						end
					end
				end
				var.pdf_cur_slot = slot;
				if slot == 0 or slot == 16 or slot == 17 or slot == 18 then
					func.pdf_CreateMenu(1);
				elseif slot == 1 or slot == 2 or slot == 3 or slot == 4 or slot == 5 or slot == 9 or slot == 15 or slot == 9 then
					func.pdf_CreateMenu(2);
				else
					func.pdf_CreateMenu(3);
				end
			end
		else
			var.pdf_cur_slot = nil;
			ui.pdf_menu:Hide();
			--_PaperDollItemSlotButton_OnEnter(var.pdf_cur_anchor);
		end
	else
		if not IsAltKeyDown() then
			var.pdf_cur_slot = nil;
			ui.pdf_menu:Hide();
		end
	end
end

function func.pdf_init()
	for i, n in pairs(slot2Name) do
		local sf = _G[n];
		local ignore_mask = sf:CreateTexture(nil, "OVERLAY");
		ignore_mask:SetAllPoints(true);
		ignore_mask:SetTexture(texture_ignore_mask);
		ignore_mask:Hide();
		ui.pdf_ignore_mask[i] = ignore_mask;
	end
	ui.pdf_menu = CreateFrame("Frame", nil, PaperDollFrame);
	ui.pdf_menu:SetFrameStrata("FULLSCREEN_DIALOG");
	ui.pdf_menu:SetScript("OnUpdate", function(self, elasped)
		if ui.pdf_menu.countingTime then
			self.countingTime = self.countingTime - elasped;
			if self.countingTime < 0 then
				self:Hide();
				self.countingTime = nil;
				var.pdf_cur_slot = nil;
				var.pdf_cur_anchor = nil;
				wipe(var.pdf_list);
			end
		end
	end);
	ui.pdf_menu:SetScript("OnEnter", function(self)
		self.countingTime = nil;
	end);
	ui.pdf_menu:SetScript("OnLeave", function(self) self.countingTime = COUNTING_TIME; end);

	func.pdf_CreateButton(1);
	ui.pdf_buttons[1]:SetPoint("TOPLEFT");

	_G.PaperDollItemSlotButton_OnEnter = function(self)
		_eventVehicle:SetScript("OnUpdate", func.pdf_eventVehicle_OnUpdate);
		if ui.pdf_menu:IsShown() then
			if var.pdf_cur_anchor == self then
				ui.pdf_menu.countingTime = nil;
			else
				var.pdf_cur_slot = nil;
				ui.pdf_menu:Hide();
			end
		end
		var.pdf_cur_anchor = self;
		--if not IsAltKeyDown() then
			return _PaperDollItemSlotButton_OnEnter(self);
		--end
	end
	_G.PaperDollItemSlotButton_OnLeave = function(self)
		var.pdf_cur_anchor = nil;
		_eventVehicle:SetScript("OnUpdate", nil);
		ui.pdf_menu.countingTime = COUNTING_TIME;
		return _PaperDollItemSlotButton_OnLeave(self);
	end
end
function func.pdf_hide_mask()
	for slot, mask in pairs(ui.pdf_ignore_mask) do
		mask:Hide();
		var.gm_ignore[slot] = false;
	end
end
function func.pdf_show_mask(set)
	for slot, mask in pairs(ui.pdf_ignore_mask) do
		if set[slot] == -1 then
			ui.pdf_ignore_mask[slot]:Show();
			var.gm_ignore[slot] = true;
		else
			ui.pdf_ignore_mask[slot]:Hide();
			var.gm_ignore[slot] = false;
		end
	end
end

local buttonBackdrop = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true,
	tileSize = 2,
	edgeSize = 2,
	insets = { left = 0, right = 0, top = 0, bottom = 0 }
};
local buttonBackdropColor = { 0.25, 0.25, 0.25, 0.75 };
local buttonBackdropBorderColor = { 0.0, 0.0, 0.0, 1.0 };

function func.OnEnter_Info(self)
	if self.info then
		GameTooltip:SetOwner(self, "ANCHOR_BOTTOMLEFT");
		for _, msg in pairs(self.info) do
			GameTooltip:AddLine(msg);
		end
		GameTooltip:Show();
	end
end
function func.OnLeave_Info(self)
	if GameTooltip:GetOwner() == self then
    	GameTooltip:Hide();
	end
end
function func.Sound_Equip()
	PlaySound(841);
end
function func.Sound_Save()
	PlaySound(841);
end
function func.Sound_Show()
	PlaySound(841);
end
function func.Sound_Hide()
	PlaySound(851);
end

StaticPopupDialogs["alaGearMan_DelSet"] = {
	text = L["Delete this outfit?"],
	button1 = L["OK"],
	button2 = L["Cancel"],
	OnShow = function(self) end,
	OnAccept = function(self) func.del(); end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 1,
}

function func.gm_CreateButton(parent, index, buttonHeight)
	-- print("CREATE", index)
	local button = CreateFrame("Button", nil, parent);
	button:SetHeight(buttonHeight);
	button:SetBackdrop(buttonBackdrop);
	button:SetBackdropColor(buttonBackdropColor[1], buttonBackdropColor[2], buttonBackdropColor[3], buttonBackdropColor[4]);
	button:SetBackdropBorderColor(buttonBackdropBorderColor[1], buttonBackdropBorderColor[2], buttonBackdropBorderColor[3], buttonBackdropBorderColor[4]);
	button:SetHighlightTexture("Interface\\FriendsFrame\\UI-FriendsFrame-HighlightBar");
	button:EnableMouse(true);
	button:Show();

	local icon = button:CreateTexture(nil, "OVERLAY");
	icon:SetTexture(texture_unk);
	icon:SetSize(buttonHeight - 4, buttonHeight - 4);
	icon:SetPoint("LEFT", 4, 0);
	button.icon = icon;

	local title = button:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall");
	title:SetPoint("LEFT", icon, "RIGHT", 4, 0);
	button.title = title;

	local glow = button:CreateTexture(nil, "OVERLAY");
	glow:SetTexture(texture_glow);
	glow:SetTexCoord(unpack(texture_glow_coord));
	glow:SetAllPoints(true);
	glow:SetBlendMode("ADD");
	glow:Hide();
	button.glow = glow;

	local delete = CreateFrame("Button", nil, button);
	delete:SetSize(buttonHeight * 0.4, buttonHeight * 0.4);
	delete:SetNormalTexture(texture_delete);
	delete:SetPoint("TOPRIGHT", -4, -2);
	delete:Show();
	delete:SetScript("OnEnter", func.OnEnter_Info);
	delete:SetScript("OnLeave", func.OnLeave_Info);
	delete:SetScript("OnClick", function(self)
		local index = button:GetDataIndex();
		if var.gm_is_editing and var.gm_editing_set == index then
			ui.custom:Hide();
		end
		var.gm_editing_set = index;
		StaticPopup_Show("alaGearMan_DelSet");
	end);
	-- delete.info = {  };
	button.delete = delete;

	local modify = CreateFrame("Button", nil, button);
	modify:SetSize(buttonHeight * 0.4, buttonHeight * 0.4);
	modify:SetNormalTexture(texture_modify);
	modify:SetPoint("BOTTOMRIGHT", -4, 2);
	modify:Show();
	modify:SetScript("OnEnter", func.OnEnter_Info);
	modify:SetScript("OnLeave", func.OnLeave_Info);
	modify:SetScript("OnClick", function(self)
		ui.custom:CustomShow(button:GetDataIndex());
	end);
	-- modify.info = {  };
	button.modify = modify;

	function button:SetIconTexture(tex)
		icon:SetTexture(tex);
	end
	function button:SetTitleText(text)
		title:SetText(text);
	end
	function button:Select()
		self.selected = true;
		self.glow:Show();
	end
	function button:Deselect()
		self.selected = nil;
		self.glow:Hide();
	end
	function button:IsSelected()
		return self.selected;
	end
	button:SetScript("OnClick", function(self)
		local dataIndex = self:GetDataIndex();
		if dataIndex > #saved_sets then
			if var.gm_cur_set then
				ui.scroll:CallButtonFuncByDataIndex(var.gm_cur_set, "Deselect");
			end
			var.gm_cur_set = nil;
			var.gm_prev_clicked_set = nil;
			func.pdf_hide_mask();
			func.new();
		elseif dataIndex == var.gm_prev_clicked_set and GetTime() - var.gm_prev_clicked_time <= 0.5 then
			var.gm_cur_set = dataIndex;
			self:Select();
			var.gm_prev_clicked_set = nil;
			func.equip();
		elseif dataIndex == var.gm_cur_set then
			var.gm_prev_clicked_set = var.gm_cur_set;
			var.gm_prev_clicked_time = GetTime();
			var.gm_cur_set = nil;
			self:Deselect();
			func.pdf_hide_mask();
		else
			if var.gm_cur_set then
				ui.scroll:CallButtonFuncByDataIndex(var.gm_cur_set, "Deselect");
			end
			var.gm_cur_set = dataIndex;
			self:Select();
			var.gm_prev_clicked_set = dataIndex;
			var.gm_prev_clicked_time = GetTime();
			func.pdf_show_mask(saved_sets[var.gm_cur_set] or {  });
		end
	end);

	button.id = index;

	return button;
end
function func.gm_SetButton(button, index)
	local sets = saved_sets;
	if index <= #sets then
		button:SetIconTexture(iconTable[sets[index].icon or 1]);
		button:SetTitleText(sets[index].name or "UNK_NAME");
		button:Show();
		if button:IsSelected() and button:GetDataIndex() ~= var.gm_cur_set then
			button:Deselect();
		elseif not button:IsSelected() and button:GetDataIndex() == var.gm_cur_set then
			button:Select();
		end
		button.delete:Show();
		button.modify:Show();
	elseif index == #sets + 1 then
		button:SetIconTexture(texture_add);
		button:SetTitleText(L["Add a new outfit"]);
		button:Show();
		if button:IsSelected() and button:GetDataIndex() ~= var.gm_cur_set then
			button:Deselect();
		elseif not button:IsSelected() and button:GetDataIndex() == var.gm_cur_set then
			button:Select();
		end
		button.delete:Hide();
		button.modify:Hide();
	else
		button:Hide();
	end
end

function func.initUI()
	ui.open = CreateFrame("Button", nil, PaperDollFrame);
	ui.open:SetSize(32, 32);
	ui.open:SetPoint("TOPRIGHT", -40, -40);
	ui.open:SetNormalTexture("interface\\paperdollinfoframe\\ui-gearmanager-button");
	ui.open:SetPushedTexture("interface\\paperdollinfoframe\\ui-gearmanager-button-pushed");
	ui.open:EnableMouse(true);
	ui.open:SetScript("OnClick", func.open_onclick);

	do	--win
		ui.gearWin = CreateFrame("Frame", nil, PaperDollFrame);
		ui.gearWin:SetFrameStrata("FULLSCREEN");
		ui.gearWin:SetBackdrop({
			bgFile = "Interface/Tooltips/UI-Tooltip-Background",
			edgeFile = "interface/dialogframe/ui-dialogbox-border",
			tile = true,
			tileSize = 16,
			edgeSize = 20,
			insets = { left = 4, right = 4, top = 4, bottom = 4 }
		});
		ui.gearWin:SetBackdropColor(0.0, 0.0, 0.0, 0.9)
		ui.gearWin:SetWidth(200);
		-- ui.gearWin:SetPoint("TOPLEFT", PaperDollFrame, "TOPRIGHT");
		-- ui.gearWin:SetPoint("BOTTOMLEFT", PaperDollFrame, "BOTTOMRIGHT");
		-- ui.gearWin:SetSize(240, 128);
		ui.gearWin:SetHeight(360);
		ui.gearWin:SetPoint("TOPLEFT", ui.open, "TOPLEFT", 45, 0);
		ui.gearWin:SetScript("OnShow", function()
			ui.scroll:SetNumValue(#saved_sets + 1);
			-- if var.gm_cur_set and saved_sets[var.gm_cur_set] then
			-- 	func.pdf_show_mask(saved_sets[var.gm_cur_set]);
			-- end
			func.Sound_Show();
		end);
		ui.gearWin:SetScript("OnHide", function(self)
			func.pdf_hide_mask();
			var.gm_cur_set = nil;
			func.Sound_Hide();
		end);
		ui.gearWin:Hide();

		ui.scroll = ALASCR(ui.gearWin, 160, ui.gearWin:GetHeight() - 64, 32, func.gm_CreateButton, func.gm_SetButton);
		ui.scroll:SetPoint("BOTTOMLEFT", 10, 10);

		ui.save = CreateFrame("Button", nil, ui.gearWin, "UIPanelButtonTemplate");
		ui.save:SetPoint("TOPLEFT", 20, -20);
		ui.save:SetText(L["Save"]);
		ui.save:SetSize(70, 24);
		ui.save:SetScript("OnClick", function(self) func.save(); end);

		ui.equip = CreateFrame("Button", nil, ui.gearWin, "UIPanelButtonTemplate");
		ui.equip:SetPoint("TOPLEFT", 110, -20);
		ui.equip:SetText(L["Equip"]);
		ui.equip:SetSize(70, 24);
		ui.equip:SetScript("OnClick", function(self) func.equip(); end);

		ui.setting = CreateFrame("Button", nil, ui.gearWin);
		ui.setting:SetSize(24, 24);
		ui.setting:SetNormalTexture(texture_modify);
		ui.setting:SetPoint("TOPRIGHT");
		ui.setting:SetScript("OnClick", func.setting);
	end

	if false then	--quick panel
		ui.quick = CreateFrame("frame");
		ui.quick:SetPoint(unpack(alaGearManSV.quickPos));
		ui.quick:EnableMouse(true);
		ui.quick:SetMovable(true);
		ui.quick:RegisterForDrag("LeftButton");
		ui.quick:SetScript("OnDragStart", function(self)
			if IsControlKeyDown() then
				self:StartMoving();
			end
		end);
		ui.quick:SetScript("OnDragStop", function(self)
			self:StopMovingOrSizing();
			self:SavePos();
		end);
		ui.quick:Show();
		function ui.quick:SavePos()
			alaGearManSV.quickPos = { self:GetPoint() };
			for k, v in pairs(alaGearManSV.quickPos) do
				if type(v) == 'table' then
					alaGearManSV.quickPos[k] = v:GetName();
				end
			end
		end
		local quickButtons = { n = 0, };
		function ui.quick:Create(index)
			local button = CreateFrame("Button", nil, ui.quick);
			button:SetSize(alaGearManSV.quickSize, alaGearManSV.quickSize);
			button:Show();
			button:EnableMouse(true);
			button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
			button:RegisterForDrag("LeftButton");
			button:SetScript("OnEnter", func.OnEnter_Info);
			button:SetScript("OnLeave", func.OnLeave_Info);
			button:SetScript("OnDragStart", function(self)
				if IsControlKeyDown() then
					ui.quick:StartMoving();
				end
			end);
			button:SetScript("OnDragStop", function(self)
				ui.quick:StopMovingOrSizing();
				ui.quick:SavePos();
			end);
			button:SetScript("OnClick", function(self, button)
				if button == "LeftButton" then
					local index = (alaGearManSV.takeoffAll_pos == 'LEFT') and (self.id - 1) or self.id;
					if index > 0 and index <= #saved_sets then
						func.equip(index);
					else
						func.takeoffAll();
					end
				else
					local elements = { };
					if alaGearManSV.quickStyle ~= 'TC' then
						tinsert(elements, {
							para = { 'quickStyle', 'TC', },
							text = L["Style_TC"],
						});
					end
					if alaGearManSV.quickStyle ~= 'T' then
						tinsert(elements, {
							para = { 'quickStyle', 'T', },
							text = L["Style_T"],
						});
					end
					if alaGearManSV.quickStyle ~= 'C' then
						tinsert(elements, {
							para = { 'quickStyle', 'C', },
							text = L["Style_C"],
						});
					end
					if alaGearManSV.takeoffAll_pos ~= 'LEFT' then
						tinsert(elements, {
							para = { 'takeoffAll_pos', 'LEFT', },
							text = L["Take-off-all On Left"],
						});
					end
					if alaGearManSV.takeoffAll_pos ~= 'RIGHT' then
						tinsert(elements, {
							para = { 'takeoffAll_pos', 'RIGHT', },
							text = L["Take-off-all On Right"],
						});
					end
					if alaGearManSV.takeoffAll_include_finger_and_trinket then
						tinsert(elements, {
							para = { 'takeoffAll_include_finger_and_trinket', false, },
							text = L["takeoffAll_include_finger_and_trinket_false"],
						});
					else
						tinsert(elements, {
							para = { 'takeoffAll_include_finger_and_trinket', true, },
							text = L["takeoffAll_include_finger_and_trinket"],
						});
					end
					ALADROP(self, "BOTTOMRIGHT", {
						handler = func.drop_handler,
						elements = elements,
					});
				end
			end)
			button:SetScript("OnMouseWheel", function(self, delta)
				if IsControlKeyDown() then
					ui.quick:SetScale(ui.quick:GetScale() + delta * 0.05);
				end
			end);
			local title = button:CreateFontString(nil, "OVERLAY");
			title:SetPoint("CENTER");
			title:SetFont(GameFontNormal:GetFont(), alaGearManSV.quickSize * 0.75, "OUTLINE");
			title:SetVertexColor(1.0, 0.75, 0.0, 1.0);
			button.title = title;
			local icon = button:CreateTexture(nil, "ARTWORK");
			icon:SetAllPoints(true);
			button.icon = icon;
			function button:SetTitleText(text)
				title:SetText(text);
			end
			function button:SetIconTexture(tex)
				icon:SetTexture(tex);
			end
			function button:ShowTitle(bool)
				if bool then
					title:Show();
				else
					title:Hide();
				end
			end
			function button:ShowIcon(bool)
				if bool then
					icon:Show();
				else
					icon:Hide();
				end
			end
			button.info = { L["CTRL-DRAG-TO-MOVE"] };
			button.id = index;
			quickButtons[index] = button;
			if index == 1 then
				button:SetPoint("TOPLEFT", 2, -2);
			else
				button:SetPoint("LEFT", quickButtons[index - 1], "RIGHT", 2, 0);
			end
			return button;
		end
		ui.quick:Create(1);
		function ui.quick:Update()
			local N = #saved_sets + 1
			if quickButtons.n > N then
				for i = max(N + 1, 1), quickButtons.n do
					quickButtons[i]:Hide();
				end
			elseif quickButtons.n < N then
				if #quickButtons < N then
					for i = #quickButtons + 1, N do
						self:Create(i);
					end
				end
				for i = quickButtons.n + 1, N do
					quickButtons[i]:Show();
				end
			end
			quickButtons.n = N;
			local TOA = (alaGearManSV.takeoffAll_pos == 'LEFT') and 1 or N;
			for i = 1, N do
				local button = quickButtons[i];
				local setIndex = i;
				if alaGearManSV.takeoffAll_pos == 'LEFT' then
					setIndex = i - 1;
				end
				if i == TOA then
					if alaGearManSV.quickStyle == 'TC' then
						button:SetTitleText(L["Take_Off_All_ButtonText"]);
						button:SetIconTexture(iconTable[37]);
						button:ShowIcon(true);
						button:ShowTitle(true);
					elseif alaGearManSV.quickStyle == 'T' then
						button:SetIconTexture(iconTable[37]);
						button:ShowIcon(true);
						button:ShowTitle(false);
					elseif alaGearManSV.quickStyle == 'C' then
						button:SetTitleText(L["Take_Off_All_ButtonText"]);
						button:SetIconTexture(texture_style_char);
						button:ShowIcon(true);
						button:ShowTitle(true);
					end
				else
					if alaGearManSV.quickStyle == 'TC' then
						button:SetTitleText(setIndex);
						button:SetIconTexture(iconTable[saved_sets[setIndex].icon or 1]);
						button:ShowIcon(true);
						button:ShowTitle(true);
					elseif alaGearManSV.quickStyle == 'T' then
						button:SetIconTexture(iconTable[saved_sets[setIndex].icon or 1]);
						button:ShowIcon(true);
						button:ShowTitle(false);
					elseif alaGearManSV.quickStyle == 'C' then
						button:SetTitleText(setIndex);
						button:SetIconTexture(texture_style_char);
						button:ShowIcon(true);
						button:ShowTitle(true);
					end
				end
			end
			ui.quick:SetSize(N * alaGearManSV.quickSize + (N + 1) * 2, alaGearManSV.quickSize + 2 * 2);
		end
		ui.quickButtons = quickButtons;
		ui.quick:Update();
		ui.quick:Hide();
	end

	do	--secure quick panel
		ui.secure = CreateFrame("frame");
		ui.secure:SetPoint(unpack(alaGearManSV.quickPos));
		ui.secure:EnableMouse(true);
		ui.secure:SetMovable(true);
		ui.secure:RegisterForDrag("LeftButton");
		ui.secure:Show();
		ui.secure:RegisterEvent("PLAYER_REGEN_ENABLED");
		ui.secure:SetScript("OnEvent", function(self, event)
			if self.OnNextSecureEnv then
				self.OnNextSecureEnv = false;
				self:Update();
			end
		end);
		function ui.secure:SavePos()
			alaGearManSV.quickPos = { self:GetPoint() };
			for k, v in pairs(alaGearManSV.quickPos) do
				if type(v) == 'table' then
					alaGearManSV.quickPos[k] = v:GetName();
				end
			end
		end
		local secureButtons = { n = 0, };
		local info = { L["CTRL-DRAG-TO-MOVE"], };
		local info_takeoffall = { L["Take_Off_All"], L["CTRL-DRAG-TO-MOVE"], };
		function ui.secure:Create(index)
			local button = CreateFrame("CheckButton", nil, ui.secure, "SecureActionButtonTemplate");
			-- button.icon:Hide();
			-- button.Flash:Hide();
			-- button.FlyoutBorder:Hide();
			-- button.FlyoutBorderShadow:Hide();
			-- button.FlyoutArrow:Hide();
			-- button.Count:Hide();
			-- button.Name:Hide();
			-- button.Border:Hide();
			-- button:SetNormalTexture(nil);
			-- button:SetPushedTexture(nil);
			button:SetAttribute('type', 'macro');
			-- button:SetAttribute('macrotext', '');
			-- SecureHandler_OnLoad(button);
			-- UnregisterStateDriver(button, "page");
			button:SetAttribute('showgrid', 0);
			-- button:SetAttribute("action", ACTION_START + index);
			button:SetSize(alaGearManSV.quickSize, alaGearManSV.quickSize);
			button:Show();
			button:EnableMouse(true);
			-- button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
			button:RegisterForDrag("LeftButton");
			button:SetScript("OnEnter", func.OnEnter_Info);
			button:SetScript("OnLeave", func.OnLeave_Info);
			button:SetScript("OnDragStart", function(self)
				if IsControlKeyDown() then
					ui.secure:StartMoving();
				end
			end);
			button:SetScript("OnDragStop", function(self)
				ui.secure:StopMovingOrSizing();
				ui.secure:SavePos();
			end);
			button:SetScript("OnMouseWheel", function(self, delta)
				if IsControlKeyDown() then
					ui.secure:SetScale(ui.secure:GetScale() + delta * 0.05);
				end
			end);
			local title = button:CreateFontString(nil, "OVERLAY");
			title:SetPoint("CENTER");
			title:SetFont(GameFontNormal:GetFont(), alaGearManSV.quickSize * 0.75, "OUTLINE");
			title:SetVertexColor(1.0, 0.75, 0.0, 1.0);
			button.title = title;
			local icon = button:CreateTexture(nil, "ARTWORK");
			icon:SetAllPoints(true);
			button.icon = icon;
			function button:SetTitleText(text)
				title:SetText(text);
			end
			function button:SetIconTexture(tex)
				icon:SetTexture(tex);
			end
			function button:ShowTitle(bool)
				if bool then
					title:Show();
				else
					title:Hide();
				end
			end
			function button:ShowIcon(bool)
				if bool then
					icon:Show();
				else
					icon:Hide();
				end
			end
			button.info = info;
			button.id = index;
			secureButtons[index] = button;
			if index == 1 then
				button:SetPoint("TOPLEFT", 2, -2);
			else
				button:SetPoint("LEFT", secureButtons[index - 1], "RIGHT", 2, 0);
			end
			return button;
		end
		ui.secure:Create(1);
		function ui.secure:Update()
			if InCombatLockdown() then
				self.OnNextSecureEnv = true;
				print(L["WAITING_FOR_REGEN_ENABLED"]);
				return;
			end
			local N = #saved_sets + 1
			if secureButtons.n > N then
				for i = max(N + 1, 1), secureButtons.n do
					secureButtons[i]:Hide();
				end
			elseif secureButtons.n < N then
				if #secureButtons < N then
					for i = #secureButtons + 1, N do
						self:Create(i);
					end
				end
				for i = secureButtons.n + 1, N do
					secureButtons[i]:Show();
				end
			end
			secureButtons.n = N;
			local TOA = (alaGearManSV.takeoffAll_pos == 'LEFT') and 1 or N;
			for i = 1, N do
				local button = secureButtons[i];
				local setIndex = i;
				if alaGearManSV.takeoffAll_pos == 'LEFT' then
					setIndex = i - 1;
				end
				if i == TOA then
					button:SetAttribute('macrotext', "/run AGM_FUNC.takeoffAll()");
					button.info = info_takeoffall;
					if alaGearManSV.quickStyle == 'TC' then
						button:SetTitleText(L["Take_Off_All_ButtonText"]);
						button:SetIconTexture(iconTable[37]);
						button:ShowIcon(true);
						button:ShowTitle(true);
					elseif alaGearManSV.quickStyle == 'T' then
						button:SetIconTexture(iconTable[37]);
						button:ShowIcon(true);
						button:ShowTitle(false);
					elseif alaGearManSV.quickStyle == 'C' then
						button:SetTitleText(L["Take_Off_All_ButtonText"]);
						button:SetIconTexture(texture_style_char);
						button:ShowIcon(true);
						button:ShowTitle(true);
					end
				else
					local macroText = "";
					local set = saved_sets[setIndex];
					for slot = 16, 18 do
						if set[slot] then
							local _, _, name = string.find(set[slot], "\124h%[(.+)%]\124h");
							if name then
								macroText = macroText .. "/equipslot [combat]" .. slot .. " " .. name .. "\n";
							end
						else
							-- macroText = macroText .. "/equipslot " .. slot .. " empty\n";
						end
					end
					macroText = macroText .. "/run AGM_FUNC.equip(" .. setIndex .. ");\n"
					button:SetAttribute('macrotext', macroText);
					--print(macroText)
					button.info = info;
					if alaGearManSV.quickStyle == 'TC' then
						button:SetTitleText(setIndex);
						button:SetIconTexture(iconTable[saved_sets[setIndex].icon or 1]);
						button:ShowIcon(true);
						button:ShowTitle(true);
					elseif alaGearManSV.quickStyle == 'T' then
						button:SetIconTexture(iconTable[saved_sets[setIndex].icon or 1]);
						button:ShowIcon(true);
						button:ShowTitle(false);
					elseif alaGearManSV.quickStyle == 'C' then
						button:SetTitleText(setIndex);
						button:SetIconTexture(texture_style_char);
						button:ShowIcon(true);
						button:ShowTitle(true);
					end
				end
			end
			ui.secure:SetSize(N * alaGearManSV.quickSize + (N + 1) * 2, alaGearManSV.quickSize + 2 * 2);
		end
		ui.secureButtons = secureButtons;
		ui.secure:Update();
		-- ui.secure:Hide();
	end

	do	--customize
		ui.custom = CreateFrame("Frame", nil, ui.gearWin);
		ui.custom:SetFrameStrata("FULLSCREEN");
		ui.custom:SetBackdrop({
			bgFile = "Interface/Tooltips/UI-Tooltip-Background",
			edgeFile = "interface/dialogframe/ui-dialogbox-border",
			tile = true,
			tileSize = 16,
			edgeSize = 20,
			insets = { left = 4, right = 4, top = 4, bottom = 4 }
		});
		ui.custom:SetBackdropColor(0.0, 0.0, 0.0, 0.9)
		ui.custom:SetSize(320, 320);
		ui.custom:SetPoint("TOPLEFT", ui.gearWin, "TOPRIGHT", 15, 0);
		ui.custom:Hide();
		ui.custom:SetScript("OnShow", function(self)
			if var.gm_editing_set and saved_sets[var.gm_editing_set] then
				var.gm_custom_icon_selected = saved_sets[var.gm_editing_set].icon or 1;
				ui.customIconButtons[var.gm_custom_icon_selected]:Select();
				ui.customEdit:SetText(saved_sets[var.gm_editing_set].name);
			else
				var.gm_custom_icon_selected = 1;
				ui.customIconButtons[var.gm_custom_icon_selected]:Select();
				ui.customEdit:SetText("");
			end
			if not ui.customEdit:GetText() or ui.customEdit:GetText() == "" then
				ui.customOK:Disable();
			end
			ui.customEdit:SetFocus();
			func.Sound_Show();
		end);
		ui.custom:SetScript("OnHide", function(self)
			ui.customIconButtons[var.gm_custom_icon_selected]:Deselect();
			var.gm_is_editing = nil;
			var.gm_editing_set = nil;
			var.gm_custom_icon_selected = nil;
			func.Sound_Hide();
		end);
		function ui.custom:CustomShow(set)
			if var.gm_editing_set == set then
				if self:IsShown() then
					self:Hide();
				else
					var.gm_is_editing = true;
					self:Show();
				end
			else
				var.gm_is_editing = true;
				var.gm_editing_set = set;
				if self:IsShown() then
					self:GetScript("OnShow")();
				else
					self:Show();
				end
			end
		end

		ui.customEdit = CreateFrame("EditBox", nil, ui.custom);
		ui.customEdit:SetSize(220, 24);
		ui.customEdit:SetFontObject(GameFontHighlightSmall);
		ui.customEdit:SetAutoFocus(false);
		ui.customEdit:SetJustifyH("LEFT");
		ui.customEdit:Show();
		ui.customEdit:EnableMouse(true);
		-- ui.customEdit:SetBackdrop({
		-- 	bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		-- 	--edgeFile = "interface/dialogframe/ui-dialogbox-border",
		-- 	tile = true,
		-- 	tileSize = 4,
		-- 	edgeSize = 0,
		-- 	insets = { left = 0, right = 0, top = 0, bottom = 0 }
		-- });
		ui.customEdit:SetPoint("TOPLEFT", ui.custom, 12, -12);
		ui.customEdit:SetScript("OnEnterPressed", function(self) self:ClearFocus(); end);
		ui.customEdit:SetScript("OnEscapePressed", function(self) self:ClearFocus(); end);
		--ui.customEdit:SetScript("OnShow", function(self) end);
		--ui.customEdit:SetScript("OnHide", function(self) end);
		ui.customEdit:SetScript("OnTextChanged", function(self, isUserInput)
			if not self:GetText() or self:GetText() == "" then
				ui.customOK:Disable();
			else
				ui.customOK:Enable();
			end
		end);
		ui.customEditTexture = ui.customEdit:CreateTexture(nil, "ARTWORK");
		ui.customEditTexture:SetPoint("TOPLEFT");
		ui.customEditTexture:SetPoint("BOTTOMRIGHT");
		ui.customEditTexture:SetTexture("Interface\\Buttons\\greyscaleramp64");
		ui.customEditTexture:SetTexCoord(0.0, 0.25, 0.0, 0.25);
		ui.customEditTexture:SetAlpha(0.75);
		ui.customEditTexture:SetBlendMode("ADD");
		ui.customEditTexture:SetVertexColor(0.25, 0.25, 0.25);

		ui.customOK = CreateFrame("Button", nil, ui.custom, "UIPanelButtonTemplate");
		ui.customOK:SetPoint("TOPRIGHT", -12, -12);
		ui.customOK:SetText(L["OK"]);
		ui.customOK:SetSize(70, 24);
		ui.customOK:SetScript("OnClick", func.customOK);

		ui.customCancel = CreateFrame("Button", nil, ui.custom, "UIPanelButtonTemplate");
		ui.customCancel:SetPoint("BOTTOMRIGHT", -12, 12);
		ui.customCancel:SetText(L["Cancel"]);
		ui.customCancel:SetSize(70, 24);
		ui.customCancel:SetScript("OnClick", func.customCancel);

		local customIconButtons = {  };
		local row = 0;
		local col = 0;
		for i = 1, #iconTable do
			local button = CreateFrame("Button", nil, ui.custom);
			button:SetSize(32, 32);
			button:SetNormalTexture(iconTable[i]);
			button:SetPoint("TOPLEFT", 15 + col * 37, - 48 - 4 - row * 37);
			button:Show();
			button:SetScript("OnClick", function(self)
				if var.gm_custom_icon_selected ~= self.id then
					if var.gm_custom_icon_selected then
						customIconButtons[var.gm_custom_icon_selected]:Deselect();
					end
					self:Select();
					var.gm_custom_icon_selected = self.id;
				end
			end);

			local glow = button:CreateTexture(nil, "OVERLAY");
			glow:SetTexture(texture_glow);
			glow:SetTexCoord(unpack(texture_glow_coord));
			glow:SetVertexColor(1.0, 0.0, 0.0);
			--glow:SetAllPoints(true);
			glow:SetPoint("BOTTOMLEFT", - 2, - 2);
			glow:SetPoint("TOPRIGHT", 2, 2);
			glow:SetBlendMode("ADD");
			glow:Hide();
			button.glow = glow;

			function button:Select()
				glow:Show();
			end
			function button:Deselect()
				glow:Hide();
			end

			button.id = i;
			customIconButtons[i] = button;
			col = col + 1;
			if col >= 8 then
				col = 0;
				row = row + 1;
			end
		end
		ui.customIconButtons = customIconButtons;
	end
end

function func.open_onclick(self, button)
	if ui.gearWin:IsShown() then
		ui.gearWin:Hide();
	else
		ui.gearWin:Show();
	end
end
function func.setting(self, button)
	local elements = { };
	if alaGearManSV.quickStyle ~= 'TC' then
		tinsert(elements, {
			para = { 'quickStyle', 'TC', },
			text = L["Style_TC"],
		});
	end
	if alaGearManSV.quickStyle ~= 'T' then
		tinsert(elements, {
			para = { 'quickStyle', 'T', },
			text = L["Style_T"],
		});
	end
	if alaGearManSV.quickStyle ~= 'C' then
		tinsert(elements, {
			para = { 'quickStyle', 'C', },
			text = L["Style_C"],
		});
	end
	if alaGearManSV.takeoffAll_pos ~= 'LEFT' then
		tinsert(elements, {
			para = { 'takeoffAll_pos', 'LEFT', },
			text = L["Take-off-all On Left"],
		});
	end
	if alaGearManSV.takeoffAll_pos ~= 'RIGHT' then
		tinsert(elements, {
			para = { 'takeoffAll_pos', 'RIGHT', },
			text = L["Take-off-all On Right"],
		});
	end
	if alaGearManSV.takeoffAll_include_finger_and_trinket then
		tinsert(elements, {
			para = { 'takeoffAll_include_finger_and_trinket', false, },
			text = L["takeoffAll_include_finger_and_trinket_false"],
		});
	else
		tinsert(elements, {
			para = { 'takeoffAll_include_finger_and_trinket', true, },
			text = L["takeoffAll_include_finger_and_trinket"],
		});
	end
	ALADROP(self, "BOTTOMRIGHT", {
		handler = func.drop_handler,
		elements = elements,
	});
end
function func.new()
	ui.custom:CustomShow();
end
function func.delete(set)
	if saved_sets[set] then
		if var.gm_cur_set == set then
			ui.scroll:CallButtonFuncByDataIndex(var.gm_cur_set, "Deselect");
			var.gm_cur_set = nil;
		end
		tremove(saved_sets, set);
		if var.gm_cur_set and var.gm_cur_set > set then
			var.gm_cur_set = var.gm_cur_set - 1;
		end
		ui.scroll:SetNumValue(#saved_sets + 1);
		-- ui.quick:Update();
		ui.secure:Update();
	end
end
function func.del()
	func.delete(var.gm_editing_set or var.gm_cur_set);
end
function func.save(set)
	set = set or var.gm_cur_set;
	if set then
		saved_sets[set] = saved_sets[set] or {  };
		local set = saved_sets[set];
		for slot = 1, 19 do
			if var.gm_ignore[slot] then
				set[slot] = -1;
			else
				local link = GetInventoryItemLink('player', slot);
				set[slot] = link;
			end
		end
		func.Sound_Save();
	end
end
function func.takeoff(slot, not_take_off_dur0)
	if not GetInventoryItemLink('player', slot) then
		return;
	end
	if not_take_off_dur0 and GetInventoryItemDurability(slot) and GetInventoryItemDurability(slot) <= 0 then
		return;
	end
	for i = 0, 4 do
		for j = 1, GetContainerNumSlots(i) do
			local cache_index = i * 100 + j;
			if not var.cache[cache_index] then
				local link = GetContainerItemLink(i, j);
				if not link then
					PickupInventoryItem(slot);
					-- PickupContainerItem(i, j);
					if i == 0 then
						PutItemInBackpack();
					else
						PutItemInBag(i + 19);
					end
					var.cache[cache_index] = true;
					C_Timer.After(1.0, function() var.cache[cache_index] = nil; end);
					return;
				end
			end
		end
	end
	if not var.cache.INVENTORY_IS_FULL then
		var.cache.INVENTORY_IS_FULL = true;
		_error_(L["INVENTORY_IS_FULL"]);
		C_Timer.After(1.0, function() var.cache.INVENTORY_IS_FULL = nil; end);
	end
end
function func.takeoffAll()
	if InCombatLockdown() then
		_error_(L["IN_COMBAT"]);
		for i = 16, 18 do
			func.takeoff(i, true);
		end
		return;
	end
	for i = 1, alaGearManSV.takeoffAll_include_finger_and_trinket and #takeoffAll_order or 12 do
		func.takeoff(takeoffAll_order[i], true);
	end
end
function func.equipItem(item, slot)
	table.wipe(var.bagItemCache);
	for i = 0, 4 do
		for j = 1, GetContainerNumSlots(i) do
			local link = GetContainerItemLink(i, j);
			if link and link == item then
				var.itemLoc:SetBagAndSlot(i, j);
				if C_Item.IsBound(var.itemLoc) then
					-- tinsert(var.bagItemCache, true);
					PickupContainerItem(i, j);
					EquipCursorItem(slot);
					return;
				else
					tinsert(var.bagItemCache, i);
					tinsert(var.bagItemCache, j);
					-- tinsert(var.bagItemCache, false);
				end
			end
		end
	end
	-- EquipItemByName(item, slot);
	if #var.bagItemCache > 0 then
		PickupContainerItem(var.bagItemCache[1], var.bagItemCache[2]);
		EquipCursorItem(slot);
	end
end
function func.equip(set)
	if InCombatLockdown() then
		_error_(L["IN_COMBAT"]);
		return;
	end
	set = set or var.gm_cur_set;
	if set and saved_sets[set] then
		local set = saved_sets[set];
		for slot = 1, 19 do
			if set[slot] ~= -1 then
				if set[slot] == nil then
					func.takeoff(slot);
				else
					if GetInventoryItemLink('player', slot) ~= set[slot] then
						func.equipItem(set[slot], slot);
					end
				end
			end
		end
		func.Sound_Equip();
	end
end
function func.customOK()
	local title = ui.customEdit:GetText();
	if title and title ~= "" then
		local index;
		if var.gm_editing_set and saved_sets[var.gm_editing_set] then
			index = var.gm_editing_set;
			saved_sets[var.gm_editing_set].name = title;
			saved_sets[var.gm_editing_set].icon = var.gm_custom_icon_selected;
		else
			index = #saved_sets + 1;
			saved_sets[index] = { name = title, icon = var.gm_custom_icon_selected, };
		end
		ui.scroll:SetNumValue(#saved_sets + 1);
		ui.scroll:Update();
		func.save(index);
		-- ui.quick:Update();
		ui.secure:Update();
	end
	func.pdf_hide_mask();
	ui.custom:Hide();
end
function func.customCancel()
	func.pdf_hide_mask();
	ui.custom:Hide();
end

function func.update_drop_table()
end
function func.drop_handler(button, key, value)
	if key == 'quickStyle' then
		if value == 'TC' or value == 'T' or value == 'C' then
			alaGearManSV.quickStyle = value;
			-- ui.quick:Update();
			ui.secure:Update();
		end
	elseif key == 'takeoffAll_pos' then
		if value == 'LEFT' or value == 'RIGHT' then
			alaGearManSV.takeoffAll_pos = value;
			-- ui.quick:Update();
			ui.secure:Update();
		end
	elseif key == 'takeoffAll_include_finger_and_trinket' then
		if type(value) == 'boolean' then
			alaGearManSV.takeoffAll_include_finger_and_trinket = value;
		end
	end
end

function func.OnEvent(self, event, arg1, ...)
	if event == "ADDON_LOADED" and arg1 == "alaGearMan" then
		self:UnregisterEvent("ADDON_LOADED");
		self:SetScript("OnEvent", nil);
		func.pdf_init();
		C_Timer.After(0.1, function()
			alaGearManSV = alaGearManSV or {  };
			alaGearManSV.sets = alaGearManSV.sets or {  };
			alaGearManSV.sets[GUID] = alaGearManSV.sets[GUID] or {  };
			saved_sets = alaGearManSV.sets[GUID];
			alaGearManSV.quickSize = alaGearManSV.quickSize or 18;
			alaGearManSV.quickStyle = alaGearManSV.quickStyle or 'C';	--'T' 'C' 'TC'
			alaGearManSV.quickPos = alaGearManSV.quickPos or { "TOP", 0, 0, };
			alaGearManSV.takeoffAll_pos = alaGearManSV.takeoffAll_pos or 'RIGHT';
			alaGearManSV.takeoffAll_include_finger_and_trinket = alaGearManSV.takeoffAll_include_finger_and_trinket or false;
			func.initUI();
		end);
	end
end
_eventVehicle:SetScript("OnEvent", func.OnEvent);

_G.AGM_FUNC = func;
_G.AGM_GVAR = global_var;

