﻿--[[--
	alex@0
--]]--
----------------------------------------------------------------------------------------------------
local ADDON, NS = ...;
----------------------------------------------------------------------------------------------------
local math, table, string, pairs, type, select, tonumber, tostring, unpack = math, table, string, pairs, type, select, tonumber, tostring, unpack;
local min, max, floor, ceil = min, max, floor, ceil;
local _G = _G;
local _ = nil;
----------------------------------------------------------------------------------------------------
local function round(v)
	local n = floor(v);
	if v - n >= 0.5 then
		return n + 1;
	else
		return n;
	end
end
----------------------------------------------------------------------------------------------------main
if alaScrollList then return; end
alaScrollList = {};
local NAME = "alaScrollList";
local function _error_(key, msg, ...)
	print("\124cffff0000" .. key .. "\124r", msg and "\124cffff0000" .. msg .. "\124r", ...);
end
--------------------------------------------------
--------------------------------------------------


function alaScrollList.CreateScrollFrame(parent, width, height, buttonHeight, funcToCreateButton, functToSetButton)
	local scrollFrame = CreateFrame("ScrollFrame", nil, parent);
	local scrollChild = CreateFrame("Frame", nil, scrollFrame);
	local scrollBar = CreateFrame("Slider", nil, scrollFrame);
	local buttons = {};
	local nButtonsShown = 0;
	local ofsIndex = 0;
	local numValue = -1;

	scrollFrame:Show();
	scrollFrame:EnableMouse(true);
	scrollFrame:SetMovable(true);
	scrollFrame:RegisterForDrag("LeftButton");
	scrollFrame:SetVerticalScroll(0);
	scrollFrame:SetScrollChild(scrollChild);
	scrollFrame:SetScript("OnMouseWheel", function(self, delta, stepSize)
		stepSize = stepSize or buttonHeight;
		local minVal, maxVal = scrollBar:GetMinMaxValues();
		scrollBar:SetValue(min(maxVal, max(minVal, scrollBar:GetValue() - delta * stepSize)));
	end);
	function scrollFrame:OnSizeChanged(width, height)
		width = width or self:GetWidth();
		height = height or self:GetHeight();
		scrollChild:SetSize(width, height);
		--scrollBar:SetValue(mscrollBar:GetValue());
		scrollChild:CreateScrollChildButtons();
		scrollFrame:Update();
	end
	scrollFrame._SetSize = scrollFrame.SetSize;
	function scrollFrame:SetSize(...)
		self:_SetSize(...);
		self:OnSizeChanged(...);
	end
	scrollFrame._SetHeight = scrollFrame.SetHeight;
	function scrollFrame:SetHeight(...)
		self:_SetHeight(...);
		self:OnSizeChange(self:GetWidth(), ...);
	end
	scrollFrame._SetWidth = scrollFrame.SetWidth;
	function scrollFrame:SetWidth(...)
		self:_SetWidth(...);
		scrollChild:SetWidth(...);
	end
	-- scrollFrame:SetScript("OnSizeChanged", function(self, width, height)
	-- 	width = width or self:GetWidth();
	-- 	height = height or self:GetHeight();
	-- 	scrollChild:SetSize(width, height);
	-- 	--scrollBar:SetValue(mscrollBar:GetValue());
	-- 	scrollChild:CreateScrollChildButtons();
	-- 	scrollFrame:Update();
	-- end);
	scrollFrame:SetScript("OnDragStart", function(self, button)
			if parent:IsMovable() then
				parent:StartMoving();
			end
		end
	);
	scrollFrame:SetScript("OnDragStop", function(self, button)
			parent:StopMovingOrSizing();
		end
	);
	scrollFrame:SetScript("OnShow", function(self)
		self:Update();
	end);
	function scrollFrame:UpdateButtons()
		for i = 1, nButtonsShown do
			functToSetButton(buttons[i], i + ofsIndex);
		end
	end
	function scrollFrame:Update()
		local maxVal = max(numValue - nButtonsShown + 2, 0) * buttonHeight;
		local val = min(scrollBar:GetValue(), maxVal);
		scrollBar:SetMinMaxValues(0, maxVal);
		scrollBar:SetValue(val);
		if nButtonsShown - 1 > numValue then
			scrollBar:Hide();
			scrollFrame:SetWidth(width + 20);
		else
			scrollBar:Show();
			scrollFrame:SetWidth(width);
		end
		scrollFrame:UpdateButtons();
	end
	function scrollFrame:SetNumValue(n)
		if n >= 0 and numValue ~= n then
			numValue = n;
			scrollFrame:Update();
		end
	end
	function scrollFrame:HandleButtonByDataIndex(index, func, ...)
		self:HandleButtonByRawIndex(index - ofsIndex, func, ...);
	end
	function scrollFrame:HandleButtonByRawIndex(index, func, ...)
		if index >= 1 and index <= #buttons then
			func(buttons[index], ...);
		else
			-- _error_("HandleButtonByRawIndex", index);
		end
	end
	function scrollFrame:CallButtonFuncByRawIndex(index, func, ...)
		if index >= 1 and index <= #buttons then
			func = buttons[index][func];
			if func then
				func(buttons[index], ...);
			else
				_error_("CallButtonFuncByRawIndex", index);
			end
		else
			-- _error_("CallButtonFuncByRawIndex", index);
		end
	end
	function scrollFrame:CallButtonFuncByDataIndex(index, func, ...)
		self:CallButtonFuncByRawIndex(index - ofsIndex, func, ...);
	end


	scrollChild:Show();
	scrollChild:SetPoint("CENTER", scrollFrame);
	function scrollChild:CreateScrollChildButtons()
		local num = ceil(self:GetHeight() / buttonHeight) + 1;
		if num == nButtonsShown then
			return;
		end
		if not buttons[1] then
			local button = funcToCreateButton(self, 1, buttonHeight);
			buttons[1] = button;
			button:SetPoint("TOPLEFT");
			button:SetPoint("TOPRIGHT");
			button:Show();
			nButtonsShown = 1;
			function button:GetDataIndex()
				return self.id + ofsIndex;
			end
		end
		if num < nButtonsShown then
			for i = num + 1, nButtonsShown do
				buttons[i]:Hide();
			end
		else
			for i = nButtonsShown + 1, num do
				local button = funcToCreateButton(self, i, buttonHeight);
				buttons[i] = button;
				button:SetPoint("TOPLEFT", buttons[i - 1], "BOTTOMLEFT", 0, 0);
				button:SetPoint("TOPRIGHT", buttons[i - 1], "BOTTOMRIGHT", 0, 0);
				buttons[i]:Show();
				function button:GetDataIndex()
					return self.id + ofsIndex;
				end
			end
		end
		nButtonsShown = num;
		--self:SetHeight(buttonHeight * nButtonsShown);
		scrollBar:SetStepsPerPage(nButtonsShown - 2);
	end


	scrollBar:SetWidth(20);
	scrollBar:SetPoint("TOPLEFT", scrollFrame, "TOPRIGHT", 0, -15);
	scrollBar:SetPoint("BOTTOMLEFT", scrollFrame, "BOTTOMRIGHT", 0, 15);
	scrollBar:Show();
	scrollBar:EnableMouse(true);
	scrollBar:SetValueStep(buttonHeight);
	scrollBar:SetMinMaxValues(0, 0);
	scrollBar:SetValue(0);
	local top = scrollBar:CreateTexture(nil, "ARTWORK");
	top:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-ScrollBar");
	top:SetSize(27, 24);
	top:SetTexCoord(0.0, 0.484375, 0.0, 0.10);
	top:SetPoint("TOPLEFT", -4, 17);
	local bot = scrollBar:CreateTexture(nil, "ARTWORK");
	bot:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-ScrollBar");
	bot:SetSize(27, 24);
	bot:SetTexCoord(0.515625, 1.0, 0.3140625, 0.4140625);
	bot:SetPoint("BOTTOMLEFT", -4, -15);
	local mid = scrollBar:CreateTexture(nil, "ARTWORK");
	mid:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-ScrollBar");
	mid:SetTexCoord(0.0, 0.484375, 0.1640625, 1.0);
	mid:SetPoint("TOPLEFT", top, "BOTTOMLEFT", 0, 0);
	mid:SetPoint("BOTTOMRIGHT", bot, "TOPRIGHT", 0, 0);
	scrollBar:SetThumbTexture("Interface\\Buttons\\UI-ScrollBar-Knob");
	local thumb = scrollBar:GetThumbTexture();
	thumb:SetSize(18, 24);
	thumb:SetTexCoord(0.20, 0.80, 0.125, 0.875);

	scrollBar:SetScript("OnValueChanged", function(self, value)
		value = value or scrollBar:GetValue();
		local index = value / buttonHeight;
		local ofs = (index - math.floor(index)) * buttonHeight;
		scrollFrame:SetVerticalScroll(ofs);
		ofsIndex = math.floor(index);
		scrollFrame:UpdateButtons();
	end);

	if width and height then
		scrollFrame:SetSize(width, height);
	end

	scrollFrame:SetNumValue(0);
	return scrollFrame;
end


_G["ALASCR"] = alaScrollList.CreateScrollFrame;
